#include "common_header.h"
#include "VertexBufferObject.h"

CVertexBufferObject::CVertexBufferObject()
{
	bDataUploaded = false;
}

/*-----------------------------------------------

Name:   CreateVBO

Params: a_iSize - initial size of buffer

Result: Creates vertex buffer object.

/*---------------------------------------------*/

void CVertexBufferObject::CreateVBO(int a_iSize)
{
	glGenBuffers(1, &uiBuffer);
	data.reserve(a_iSize);
	iSize = a_iSize;
}

/*-----------------------------------------------

Name:   ReleaseVBO

Params: none

Result: Releases VBO and frees all memory.

/*---------------------------------------------*/

void CVertexBufferObject::ReleaseVBO()
{
	glDeleteBuffers(1, &uiBuffer);
	bDataUploaded = false;
	data.clear();
}

/*-----------------------------------------------

Name:   BindVBO

Params: a_iBufferType - buffer type (GL_ARRAY_BUFFER, ...)

Result: Binds this VBO.

/*---------------------------------------------*/

void CVertexBufferObject::BindVBO(int a_iBufferType)
{
	iBufferType = a_iBufferType;
	glBindBuffer(iBufferType, uiBuffer);
}

/*-----------------------------------------------

Name:   UploadDataToGPU

Params: iUsageHint - GL_STATIC_DRAW, GL_DYNAMIC_DRAW...

Result: Sends data to GPU.

/*---------------------------------------------*/

void CVertexBufferObject::UploadDataToGPU(int iDrawingHint)
{
	glBufferData(iBufferType, data.size(), &data[0], iDrawingHint);
	bDataUploaded = true;
	data.clear();
}

/*-----------------------------------------------

Name:   AddData

Params: ptrData - pointer to arbitrary data
        uiDataSize - data size in bytes

Result: Adds arbitrary data to VBO.

/*---------------------------------------------*/

void CVertexBufferObject::AddData(void* ptrData, UINT uiDataSize)
{
	data.insert(data.end(), (BYTE*)ptrData, (BYTE*)ptrData+uiDataSize);
}
