#pragma once

/********************************

Class:   COpenGLControl

Purpose: Provides convenient usage
         of OpenGL

********************************/

class COpenGLControl
{
public:
	COpenGLControl() { bGlewInitialized = false; iFPSCount = 0; iCurrentFPS = 0; }

	bool InitOpenGL(SDL_Window * a_window, void (*a_InitScene)(void *), void (*a_RenderScene)(void *), void(*a_ReleaseScene)(void *), void * lpParam);
	bool InitSDL2(int iMajorVersion, int iMinorVersion);

	void ResizeOpenGLViewportFull();
	void SetProjection3D(float fFOV, float fAspectRatio, float fNear, float fFar);
	void SetOrtho2D(int width, int height);

	glm::mat4* GetProjectionMatrix();
	glm::mat4* GetOrthoMatrix();

	void Render(void * lpParam);
	void ReleaseOpenGLControl(void * lpParam);

	void SwapBuffersM();

	bool SetVerticalSynchronization(int interval);

	int GetFPS();

	int GetViewportWidth();
	int GetViewportHeight();

private:
	SDL_Window * window;
	SDL_GLContext context;
	bool InitGLEW();

	bool bGlewInitialized;

	// Used for FPS calculation
	int iFPSCount, iCurrentFPS;
	unsigned int tLastSecond;

	// Matrix for perspective projection
	glm::mat4 mProjection;
	// Matrix for orthographic 2D projection
	glm::mat4 mOrtho;

	// Viewport parameters
	int iViewportWidth, iViewportHeight;

	void (*InitScene)(void * lpParam), (*RenderScene)(void * lpParam), (*ReleaseScene)(void * lpParam);
};
