#pragma once

enum ETextureFiltering
{
	TEXTURE_FILTER_MAG_NEAREST = 0, // Nearest criterion for magnification
	TEXTURE_FILTER_MAG_BILINEAR, // Bilinear criterion for magnification
	TEXTURE_FILTER_MIN_NEAREST, // Nearest criterion for minification
	TEXTURE_FILTER_MIN_BILINEAR, // Bilinear criterion for minification
	TEXTURE_FILTER_MIN_NEAREST_MIPMAP, // Nearest criterion for minification, but on closest mipmap
	TEXTURE_FILTER_MIN_BILINEAR_MIPMAP, // Bilinear criterion for minification, but on closest mipmap
	TEXTURE_FILTER_MIN_TRILINEAR, // Bilinear criterion for minification on two closest mipmaps, then averaged
};

/********************************

Class:   CTexture

Purpose: Wraps OpenGL texture
         object and performs
         their loading.

********************************/

class CTexture
{
public:
	void CreateFromData(BYTE* bData, int a_iWidth, int a_iHeight, GLenum format, bool bGenerateMipmaps);
	bool LoadTexture2D(std::string a_sPath, bool bGenerateMipmaps = false);
	void BindTexture(int iTextureUnit = 0);

	void SetFiltering(int a_tfMagnification, int a_tfMinification);

	void SetSamplerParameter(GLenum parameter, GLenum value);

	int GetMinificationFilter();
	int GetMagnificationFilter();

	void ReleaseTexture();

	CTexture();
private:
	UINT uiTexture; // Texture name
	UINT uiSampler; // Sampler name
	bool bMipmapsGenerated;

	int tfMinification, tfMagnification;

	std::string sPath;
};
